<?php

namespace Drupal\taxonomy_vocabulary_tree\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\taxonomy\TermInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block.
 *
 * @Block(
 *  id = "taxonomy_vocabulary_tree_block",
 *  admin_label = @Translation("Taxonomy vocabulary tree block"),
 * )
 */
class SliderNamesTreeBlock extends BlockBase implements ContainerFactoryPluginInterface {

    /**
     * The taxonomy term storage.
     *
     * @var \Drupal\taxonomy\TermStorageInterface
     */
    protected $termStorage;

    /**
     * The renderer.
     *
     * @var \Drupal\Core\Render\RendererInterface
     */
    protected $renderer;

    /**
     * The language manager.
     *
     * @var \Drupal\Core\Language\LanguageManagerInterface
     */
    protected $languageManager;

    /**
     * Constructs a new SystemBreadcrumbBlock object.
     *
     * @param array $configuration
     *   A configuration array containing information about the plugin instance.
     * @param string $plugin_id
     *   The plugin_id for the plugin instance.
     * @param mixed $plugin_definition
     *   The plugin implementation definition.
     * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
     *   The entity type manager.
     * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
     *   The language manager.
     *
     * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
     * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
     */
    public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        EntityTypeManagerInterface $entity_type_manager,
        LanguageManagerInterface $language_manager
    ) {
        parent::__construct($configuration, $plugin_id, $plugin_definition);
        $this->termStorage = $entity_type_manager->getStorage(
            'taxonomy_term'
        );
        $this->languageManager = $language_manager;

    }

    /**
     * {@inheritdoc}
     *
     * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
     * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
     */
    public static function create(
        ContainerInterface $container,
        array $configuration,
        $plugin_id,
        $plugin_definition
    ) {
        /*  @noinspection  PhpParamsInspection */
        return new static(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $container->get('entity_type.manager'),
            $container->get('language_manager')
        );
    }

    /**
     * {@inheritdoc}
     *
     * @throws \Exception
     */
    public function build() {
        $config = $this->getConfiguration();
        if (empty($config['vocabulary_name'])) {
            return $this->warningsMessage();
        }

        $tree = $this->loadTree($config['vocabulary_name']);
        $structured_tree = $this->structuringTree($tree);
        $build = $this->buildTerms($structured_tree);
        $build['#attached']['library'][] = 'taxonomy_vocabulary_tree/menu-tree';
        $build['#attributes']['class'][] = 'taxonomy-menu-tree';

        return $build;
    }

    /**
     * Load vocabulary tree.
     *
     * @param string $vocabulary_name
     *   The vocabulary name.
     *
     * @return array
     *   The array with terms tree.
     */
    protected function loadTree(string $vocabulary_name) {
        $terms = $this->termStorage->loadTree($vocabulary_name, 0, NULL, TRUE);
        $langcode = $this->languageManager->getCurrentLanguage()->getId();

        return $this->getTranslations($terms, $langcode);
    }

    /**
     * Structuring vocabulary tree.
     *
     * @param array $terms
     *   The terms.
     *
     * @return array
     *   The structured tree.
     */
    protected function structuringTree(array $terms) {
        $tree = [];
        $depth = 0;
        $links = [0 => &$tree];
        $link = $links[0];
        foreach ($terms as $term) {
            $parent_id = $term->get('parent')->getString();
            if ($term->depth > $depth) {
                $link = &$link['children'][$parent_id];
                $links[$parent_id] = &$link;
            }
            $depth = $term->depth;

            unset($link);
            $link = &$links[$parent_id];
            $link['children'][$term->id()]['entity'] = $term;
        }

        return $tree['children'];
    }

    /**
     * Translating terms.
     *
     * @param array $terms
     * @param string $langcode
     *
     * @return \Drupal\taxonomy\Entity\Term[]
     *   The translated terms.
     */
    protected function getTranslations(array $terms, string $langcode) {
        $translated_terms = [];

        /** @var \Drupal\taxonomy\Entity\Term $term */
        foreach ($terms as $term) {
            if ($term->hasTranslation($langcode)) {
                $translated_terms[$term->id()] = $term->getTranslation($langcode);
            }
            else {
                $translated_terms[$term->id()] = $term;
            }
        }

        return $translated_terms;
    }

    /**
     * Show warning message.
     *
     * @return array
     *   The status messages.
     */
    protected function warningsMessage() {
        $this->messenger()->addWarning(
            $this->t('Please select vocabulary_name in configuration block.')
        );

        return [
            '#type' => 'status_messages',
        ];
    }

    /**
     * Build terms.
     *
     * @param array $terms
     *   The term.
     *
     * @return array
     *   The renderable array.
     * @throws \Drupal\Core\Entity\EntityMalformedException
     */
    protected function buildTerms(array $terms) {
        $menu_classes = $this->getConfiguration()['menu_classes'] ?? '';

        $build = [
            '#type' => 'container',
            '#theme_wrappers' => [],
            '#prefix' => '<ul class="' . $menu_classes . ' menu-tree-main-level menu-tree-nested">',
            '#suffix' => '</ul>',
        ];

        foreach ($terms as $term) {
            $build[] = $this->buildTerm($term);
        }

        return $build;
    }

    /**
     * Build term.
     *
     * @param array $term
     *   The term.
     *
     * @return array
     *   Renderable array.
     * @throws \Drupal\Core\Entity\EntityMalformedException
     */
    protected function buildTerm($term) {
        $config = $this->getConfiguration();
        $build = [
            '#type' => 'inline_template',
            '#template' => '<li class="{{ item_classes }} without-nesting"> {{ html_prefix }}  {{ term_name }} </li>',
            '#context' => [
                'item_classes' => $config['item_classes'] ?? '',
                'html_prefix' => [
                    '#markup' => $config['without_nesting_html_prefix'] ?? '',
                ],
                'term_name' => [
                    '#markup' => $this->buildTermName($term['entity']),
                ],
            ],
        ];

        if (empty($term['children'])) {
            return $build;
        }
        else {
            $children = [];
            foreach ($term['children'] as $child) {
                $children[] = $this->buildTerm($child);
            }

            $build['#template'] = '<li class="{{ item_classes }}"> {{ html_prefix }}  {{ term_name }} <ul class="menu-tree-nested">';
            $build['#context']['html_prefix']['#markup'] = $config['html_prefix'] ?? '';

            return [
                '#type' => 'container',
                '#theme_wrappers' => [],
                'item' => $build,
                'children' => $children,
                '#suffix' => '</ul></li>',
            ];
        }
    }

    /**
     * Build term name.
     *
     * @param \Drupal\taxonomy\TermInterface $term
     *   The term.
     *
     * @return string
     *   The term name link.
     *
     * @throws \Drupal\Core\Entity\EntityMalformedException
     */
    protected function buildTermName(TermInterface $term) {
        $config = $this->getConfiguration();
        $use_url_from_term = $config['use_url_from_term'] ?? FALSE;

        if ($use_url_from_term) {
            $term_url = $term->toUrl()->toString();
        }
        else {
            $url_template = $this->getConfiguration()['term_url'] ?? '';
            $term_url = str_replace('[tid]', $term->id(), $url_template);
        }


        return '<a href="' . $term_url . '">' . $term->getName() . '</a>';
    }

    /**
     * {@inheritdoc}
     */
    public function blockForm($form, FormStateInterface $form_state) {
        $form = parent::blockForm($form, $form_state);
        $vocabulary_names = taxonomy_vocabulary_get_names();

        $config = $this->getConfiguration();
        $form['vocabulary_name'] = [
            '#type' => 'select',
            '#title' => $this->t('Vocabulary name'),
            '#options' => $vocabulary_names,
            '#default_value' => $config['vocabulary_name'] ?? NULL,
            '#required' => TRUE,
        ];

        $form['use_url_from_term'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Use url from term entity.'),
        ];

        $form['term_url'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Term url'),
            '#default_value' => $config['term_url'] ?? '/taxonomy/term/[tid]',
            '#maxlength' => 255,
            '#description' => $this->t('You can use [tid] for term id in url.'),
            '#required' => TRUE,
            '#states' => [
                'visible' => [
                    ':input[name="settings[use_url_from_term]"]' => [
                        'checked' => FALSE
                    ],
                ],
            ],
        ];

        $form['html_prefix'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Html prefix'),
            '#default_value' => $config['html_prefix'] ?? '<i class="menu-tree-caret"></i>',
            '#maxlength' => 255,
            '#description' => $this->t('The html prefix before menu item.'),
        ];

        $form['without_nesting_html_prefix'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Without nesting html prefix'),
            '#default_value' => $config['without_nesting_html_prefix'] ?? '<i class="menu-tree-caret-without-nesting"></i>',
            '#maxlength' => 255,
            '#description' => $this->t('The html prefix before menu item without nesting.'),
        ];

        $form['menu_classes'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Menu classes'),
            '#default_value' => $config['menu_classes'] ?? 'list-group categories',
            '#maxlength' => 255,
            '#description' => $this->t('The css classes of menu.'),
        ];

        $form['item_classes'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Menu item classes'),
            '#default_value' => $config['item_classes'] ?? 'list-group-item',
            '#maxlength' => 255,
            '#description' => $this->t('The css classes of menu item.'),
        ];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function blockSubmit($form, FormStateInterface $form_state) {
        foreach ($form_state->getValues() as $name => $value) {
            $this->setConfigurationValue($name, $value);
        }
    }

}
